import torch
import torch.nn as nn
import torchvision
import torchvision.transforms as transforms
import model_n_uae as model
from torch.autograd import Variable
import med_dataset_wsl as med_dataset
from torchvision.utils import save_image
import os
import numpy as np

from PIL import Image
from pylab import *
import imageio
import time


def mergeout(x):
    ori_image = x.reshape(x.size(0),3,240,240)
    return ori_image

def mergepic(x):
    ori_image = x.reshape(x.size(0),1,240,240)
    return ori_image

def rewrite(x):
    y = (x > 127).astype('uint8')
    return y

def test(pic1,pic2):
    picc1 = pic1#[0,:,:,:]
    picc2 = pic2#[0,:,:,:]
    if picc1.shape[2] == picc2.shape[2]:
        N = picc1.shape[2]

        picc1 = rewrite(picc1)
        picc2 = rewrite(picc2)

        picc1_flat = picc1.reshape(-1,N)
        picc1_flat = picc1_flat.T
        picc2_flat = picc2.reshape(-1,N)
        picc2_flat = picc2_flat.T

        intersection = picc1_flat * picc2_flat
        T = picc2_flat.sum(1)

        P = picc1_flat.sum(1)
        TP = intersection.sum(1)
        FP = P-TP
        FN = T-TP

        dice = ((2*TP) +1) / (T+P+1)
        dice_loss = 1 - dice.sum()/N

        ppv = (TP+1) / (TP+FP+1)
        ppv_loss = 1 - ppv.sum()/N

        sensitivity = (TP+1) / (TP+FN+1)
        sensitivity_loss = 1 - sensitivity.sum()/N

        iou = (TP+1) / (T+P-TP +1)
        iou_loss = 1 - iou.sum()/N

    else:
        print('size dont match')

    return dice,dice_loss,ppv,ppv_loss,sensitivity,sensitivity_loss,iou,iou_loss

pth_filepath_u = '/mnt/sdb/yd2tb/yd2tianbiao_checkpoints/nuae_2023-02-15_14:15:21_2_0.0003_1111/9_ua_MODEL.pth'
pth_filepath_ae = '/mnt/sdb/yd2tb/yd2tianbiao_checkpoints/nuae_2023-02-15_14:15:21_2_0.0003_1111/9_uae_MODEL.pth'
device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
batch_size = 1
log_path = './Results/nuae_2023-02-15_14:15:21_2_0.0003_1111'
folder_name = "{0}_{1}".format('uae','epoch_9')

trans = transforms.ToTensor()
custom_dataset = med_dataset.med('./data/test/train','./data/test/anno',
                                transform = trans)

test_dataset = torch.utils.data.DataLoader(dataset=custom_dataset,
                                            batch_size = batch_size,
                                            shuffle = False)
                                            
net_u = model.NestedUNet(3).to(device)
net_ae = model.AE(3).to(device)
net_u.load_state_dict(torch.load(pth_filepath_u, map_location='cuda:0'))
net_ae.load_state_dict(torch.load(pth_filepath_ae, map_location='cuda:0'))
print('load_done')

for i, (imagei,imagel,imagea) in enumerate(test_dataset):

    imagei = imagei.type(torch.FloatTensor)
    imagei = imagei.to(device)
    imagea = imagea.type(torch.FloatTensor)
    imagea = imagea.to(device)
    imagel = imagel.to(device)

    out, interout = net_u(imagei)
    _, _, ori = net_ae(imagei)
    mask, intermask, _ = net_ae(imagea)

    #创建文件夹保存运行结果
    if not os.path.exists(os.path.join(log_path,folder_name,'SEG_GT')):
        os.makedirs(os.path.join(log_path,folder_name,'SEG_GT'))
    if not os.path.exists(os.path.join(log_path,folder_name,'SEG_OUT')):
        os.makedirs(os.path.join(log_path,folder_name,'SEG_OUT'))
    if not os.path.exists(os.path.join(log_path,folder_name,'ORI_GT')):
        os.makedirs(os.path.join(log_path,folder_name,'ORI_GT'))
    if not os.path.exists(os.path.join(log_path,folder_name,'ORI_OUT')):
        os.makedirs(os.path.join(log_path,folder_name,'ORI_OUT'))
    if not os.path.exists(os.path.join(log_path,folder_name,'MASK_OUT')):
        os.makedirs(os.path.join(log_path,folder_name,'MASK_OUT'))

    oriout_images = mergeout(imagel)
    save_image(oriout_images, os.path.join(log_path,folder_name,'SEG_GT','image-{}.png'.format(i+1)))
    out_images = mergeout(out)
    save_image(out_images,os.path.join(log_path,folder_name,'SEG_OUT','image-{}.png'.format(i+1)))
    ori_images = mergepic(imagei)
    save_image(ori_images, os.path.join(log_path,folder_name,'ORI_GT','image-{}.png'.format(i+1)))
    oriae_images = mergepic(ori)
    save_image(oriae_images,os.path.join(log_path,folder_name,'ORI_OUT','image-{}.png'.format(i+1)))
    outae_images = mergeout(mask)
    save_image(outae_images, os.path.join(log_path,folder_name,'MASK_OUT','image-{}.png'.format(i+1)))

    if i%100 == 0:
        B = "Step[{}/{}]".format(i+1,len(test_dataset))
        print(B)


print('metric')
root = os.path.join(log_path,folder_name)
dir1 = os.path.join(root,'SEG_OUT')
dir2 = os.path.join(root,'SEG_GT')

dice_ = []
ppv_ = []
sensi_ = []
iou_ = []
dices_ = []
ppvs_ = []
sensis_ = []
ious_ = []

j = 0
list = os.listdir(dir1)[:]

for i in list:
    if i.endswith('.png'):
        path1 = os.path.join(dir1,i)
        path2 = os.path.join(dir2,i)

        pic1 = imageio.imread(path1)
        pic2 = imageio.imread(path2)
        dice_s,dice,ppv_s,ppv,sensi_s,sensi, iou_s,iou = test(pic1,pic2)

        dice_.append(dice)
        ppv_.append(ppv)
        sensi_.append(sensi)
        iou_.append(iou)
        dices_.append(dice_s)
        ppvs_.append(ppv_s)
        sensis_.append(sensi_s)
        ious_.append(iou_s)

        j += 1

        with open(os.path.join(root,'Log_metric.txt'), 'a') as log:
            log.write("Step: {}\n".format(j))
            log.write("Dice: {:.4f}, PPV:{:.4f}, Sensi:{:.4f}, IoU:{:.4f}\n".format((1-dice)*100,(1-ppv)*100,(1-sensi)*100,(1-iou)*100))

        if j%8 == 0:
            dice_a = sum(dice_)/j
            ppv_a = sum(ppv_)/j
            sensi_a = sum(sensi_)/j
            iou_a = sum(iou_)/j

            B = "Step[{}/{}],Dice:{:.4f},PPV:{:.4f},Sensi:{:.4f},IoU:{:.4f}\n".format(j,len(list),(1-dice_a)*100,(1-ppv_a)*100,(1-sensi_a)*100,(1-iou_a)*100)
            print(B)
            with open(os.path.join(root,'Log_metric.txt'), 'a') as log:
                log.write(B)

            if j == len(list):
                Three = (1-dice_a)*100 + (1-ppv_a)*100 + (1-sensi_a)*100
                Toal = Three + (1-iou_a)*100

                C = "Step[{}/{}],Dice:{:.4f},PPV:{:.4f},Sensi:{:.4f},IoU:{:.4f},Toal:{:.4f},Three:{:.4f}\n".format(j,len(list),(1-dice_a)*100,(1-ppv_a)*100,(1-sensi_a)*100,(1-iou_a)*100,Toal,Three)
                print(C)
                with open(os.path.join(log_path,'metric.txt'), 'a') as log:
                    log.write(C)

        np.save(os.path.join(root,'test_dice_list.npy'),dice_)
        np.save(os.path.join(root,'test_ppv_list.npy'),ppv_)
        np.save(os.path.join(root,'test_sensi_list.npy'),sensi_)
        np.save(os.path.join(root,'test_iou_list.npy'),iou_)
        np.save(os.path.join(root,'test_dice_array'),dices_)
        np.save(os.path.join(root,'test_ppv_array.npy'),ppvs_)
        np.save(os.path.join(root,'test_sensi_array.npy'),sensis_)
        np.save(os.path.join(root,'test_iou_array.npy'),ious_)
