

import cv2
import numpy as np
import torch
from torch.autograd import Variable
from torchvision import models
import model_nestedwsl as model
import med_dataset

import torch.nn as nn
import torchvision
import torchvision.transforms as transforms
import datetime
from torch.autograd import Variable
from torchvision.utils import save_image
import os

def mergeout(x):
    ori_image = x.reshape(x.size(0),3,240,240)
    return ori_image

def mergepic(x):
    ori_image = x.reshape(x.size(0),1,240,240)
    return ori_image

def rewrite(x):
    y = (x > 127).astype('uint8')
    return y

pth_filepath = './checkpoints/nestedwsl_2020-04-22_21:45:47_4_70_0.0003_1117/35_nestedwsl_MODEL.pth'
device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
log_path_image = './a/image'
log_path_mask = './a/mask'

trans = transforms.ToTensor()

net = model.Nestedwsl(3).to(device)
net = torch.nn.DataParallel(net,device_ids=[0,1])
net.load_state_dict(torch.load(pth_filepath, map_location='cuda:0'))

imagei = './image.jpg'
imagea = './mask.jpg'
imagei = trans(imagei)
imagea = trans(imagea)
output, ori, x0_0, x1_0, x2_0, x3_0, x4_0, x3_1, x2_2, x1_3, x0_4, x4_0_ae, x3_1_ae, x2_2_ae, x1_3_ae, x0_4_ae = net(imagei)
output_a, _, x0_0_a, x1_0_a, x2_0_a, x3_0_a, x4_0_a, x3_1_a, x2_2_a, x1_3_a, x0_4_a, _, _, _, _, _ = net(imagea)

def preprocess_image(cv2im, resize_im=True):

    # mean and std list for channels (Imagenet)
    mean = [0.485, 0.456, 0.406]
    std = [0.229, 0.224, 0.225]
    # Resize image
    if resize_im:
        cv2im = cv2.resize(cv2im, (240, 240))
    im_as_arr = np.float32(cv2im)
    im_as_arr = np.ascontiguousarray(im_as_arr[..., ::-1])
    im_as_arr = im_as_arr.transpose(2, 0, 1)  # Convert array to D,W,H
    # Normalize the channels
    for channel, _ in enumerate(im_as_arr):
        im_as_arr[channel] /= 255
        im_as_arr[channel] -= mean[channel]
        im_as_arr[channel] /= std[channel]
    # Convert to float tensor
    im_as_ten = torch.from_numpy(im_as_arr).float()
    # Add one more channel to the beginning. Tensor shape = 1,3,224,224
    im_as_ten.unsqueeze_(0)
    # Convert to Pytorch variable
    im_as_var = Variable(im_as_ten, requires_grad=True)
    return im_as_var


class FeatureVisualization():
    def __init__(self,img_path,selected_layer):
        self.img_path=img_path
        self.selected_layer=selected_layer
        self.pretrained_model = net.features

    def process_image(self):
        img=cv2.imread(self.img_path)
        img=preprocess_image(img)
        return img

    def get_feature(self):
        input=self.process_image()
        #print(input.shape)
        x=input
        for index,layer in enumerate(self.pretrained_model):
            x=layer(x)
            if (index == self.selected_layer):
                return x

    def get_single_feature(self):
        features=self.get_feature()
        print(features.shape)

        feature=features[:,0,:,:]
        print(feature.shape)

        feature=feature.view(feature.shape[1],feature.shape[2])
        print(feature.shape)

        return feature

    def save_feature_to_img(self):
        #to numpy
        feature=self.get_single_feature()
        feature=feature.data.numpy()

        #use sigmod to [0,1]
        feature= 1.0/(1+np.exp(-1*feature))

        # to [0,255]
        feature=np.round(feature*255)
        print(feature[0])

        cv2.imwrite('./img.jpg',feature)




if __name__=='__main__':
    # get class
    myClass=FeatureVisualization('./image.jpg',5)
    #print (myClass.pretrained_model)

    myClass.save_feature_to_img()
